use leptos::prelude::*;

pub fn shell(options: LeptosOptions) -> impl IntoView {
    use leptos_meta::MetaTags;

    view! {
        <!DOCTYPE html>
        <html lang="en">
            <head>
                <meta charset="utf-8" />
                <meta name="viewport" content="width=device-width, initial-scale=1" />
                <AutoReload options=options.clone() />
                <HydrationScripts options />
                <MetaTags />
            </head>
            <body>
                <App />
            </body>
        </html>
    }
}

#[leptos::component]
pub fn App() -> impl leptos::IntoView {
    use leptos_meta::Stylesheet;
    use leptos_meta::Title;

    #[cfg(feature = "ssr")]
    tracing::debug!("Rendering root component");

    // Provides context that manages stylesheets, titles, meta tags, etc.
    leptos_meta::provide_meta_context();

    leptos::view! {
        <Stylesheet id="leptos" href="/pkg/ascend.css" />

        // sets the document title
        <Title text="Ascend" />

        <main>
            <Ascend />
        </main>
    }
}

#[leptos::component]
fn Ascend() -> impl leptos::IntoView {
    leptos::view! { <crate::pages::wall::Wall /> }
}
